from __future__ import with_statement
from direct.gui.OnscreenText import OnscreenText
from pandac.PandaModules import TextNode,Point3
from time import gmtime,time
import math,os

__author__='sigilis.com'
__version__='0.2.6'

# Function to put instructions on the screen.
def addInstructions(pos, msg, wscreen):
	if wscreen=='yes':
		xpos=-1.5
	else:
		xpos=-1.3
	ost=OnscreenText(text=msg, style=1, fg=(1,1,1,1),
			pos=(xpos, pos), align=TextNode.ALeft, scale = .05)
	ost.reparentTo(aspect2d)
	return ost

# Function to put title on the screen.
def addTitle(text):
    ost=OnscreenText(text=text, style=1, fg=(1,1,1,1),
	                pos=(1,-0.95), align=TextNode.ARight, scale = .07)
    ost.reparentTo(aspect2d)
    return ost
    
def addSplashLine(pos, msg):
    ost=OnscreenText(text=msg, style=1, fg=(1,1,1,1),
			pos=(0, pos), align=TextNode.ACenter, scale = .05) 			
    ost.reparentTo(aspect2d)
    return ost
    
def getSunPos():
	""" 	Red Sun rises in East sets in West (sector 1) y=0
		Green Sun Rises in North sets in South (sector1) x=0
		Blue Sun circles world equator z=0
		
		periods:
		red sun: 	23311 s		49 889 664 km
		green sun: 	33161 s		77 248 512 km
		blue sun:	44711 s		94 951 296 km
		
	"""
	rSUNDISTANCE=5280*31000000
	gSUNDISTANCE=5280*48000000
	bSUNDISTANCE=5280*59000000
	DTIME=time()
	rAngle=((DTIME % 23311)/23311)*(math.pi*2.0)
	gAngle=((DTIME % 33161)/33161)*(math.pi*2.0)
	bAngle=((DTIME % 44711)/44711)*(math.pi*2.0)
	print 'ANGLES',rAngle,gAngle,bAngle
	rSun=.5+.5*math.sin(rAngle)
	gSun=.5+.5*math.sin(gAngle)
	bSun=.5
	#bSun=0
	return (Point3(math.cos(rAngle)*rSUNDISTANCE,0,math.sin(rAngle)*rSUNDISTANCE),
		Point3(0,-math.cos(gAngle)*gSUNDISTANCE,math.sin(gAngle)*gSUNDISTANCE),
		Point3(-math.cos(bAngle)*bSUNDISTANCE,math.sin(bAngle)*bSUNDISTANCE,0),
		Point3(rSun,gSun,bSun) )
	
def checkForFile(fpath):
	if os.path.exists(fpath):
		return True
	else:	
		return False
		
def backUpFile(fpath):
	try:
		nowtime=time()
		print 'nowtime',nowtime
		nowtime=str(nowtime).split('.')
		back=open(fpath+nowtime[0],'w')	
		with open(fpath) as f:
			for line in f:
				back.write(line)
		back.close()	
		return True
	except:
		return False
		
def writeFile(fpath,newContent):
	try:
		nfile=open(fpath,'w')
		nfile.write(newContent)
		nfile.close()
		return True
	except:
		return False
