__author__='sigilis.com'
__version__='0.1.1'
from pandac.PandaModules import *
import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from direct.distributed.PyDatagram import PyDatagram
from direct.distributed.PyDatagramIterator import PyDatagramIterator

from direct.gui.DirectGui import *
import sys

import threading


######################################
##
## Config
##

IP = 'sigilis.com'
PORT = 9099


######################################3
##
## Defines
##


MSG_NONE            = 0
CMSG_AUTH           = 1
SMSG_AUTH_RESPONSE  = 2
CMSG_CHAT           = 3
SMSG_CHAT           = 4
CMSG_DISCONNECT_REQ = 5
SMSG_DISCONNECT_ACK = 6
CMSG_POS	    = 7
SMSG_POS_RESPONSE   = 8
CMSG_MODEL	    = 9
SMSG_MODEL 	    = 10
CMSG_USER	    = 11
SMSG_USER	    = 12





class Client(DirectObject):
	def __init__(self,user=None):
		print "init client"
		self.connected=False
		self.user=user
		self.Handlers = {
			SMSG_AUTH_RESPONSE  : self.msgAuthResponse,
			SMSG_CHAT           : self.msgChat,
			SMSG_DISCONNECT_ACK : self.msgDisconnectAck,
			SMSG_POS_RESPONSE   : self.msgPosResponse,
			SMSG_MODEL	    : self.msgSectorModel,
			SMSG_USER	    : self.msgSectorUser
		}	
		self.accept("escape", self.sendMsgDisconnectReq)
		
		# Create network layer objects
		self.cManager = QueuedConnectionManager()
		self.cListener = QueuedConnectionListener(self.cManager, 0)
		self.cReader = QueuedConnectionReader(self.cManager, 0)
		self.cWriter = ConnectionWriter(self.cManager,0)
		print IP, PORT
		self.Connection = self.cManager.openTCPClientConnection(IP, PORT,1)
		if (self.Connection != None):
			self.cReader.addConnection(self.Connection)
	
			# Start tasks
			taskMgr.add(self.readTask, "serverReaderPollTask", -39)
	
			# Send login msg to the server
			self.sendMsgAuth()

	def readTask(self, Task):
	    	while 1:
			(datagram, data, msgID) = self.nonBlockingRead(self.cReader)
			if msgID is MSG_NONE:
				break
			else:
				self.handleDatagram(data, msgID)
                return Task.cont

	def nonBlockingRead(self,qcr):
		"""
		Return a datagram iterator and type if data is available on the
		queued connection reader
		"""
        	if self.cReader.dataAvailable():
			datagram = NetDatagram()
			if self.cReader.getData(datagram):
				data = PyDatagramIterator(datagram)
				msgID = data.getUint16()
			else:
			    data = None
			    msgID = MSG_NONE
		else:
			datagram = None
			data = None
            		msgID = MSG_NONE
		# Note, return datagram to keep a handle on the data
		return (datagram, data, msgID)

	def handleDatagram(self, data, msgID):
		"""
		Check if there's a handler assigned for this msgID.
		Since we dont have case statements in python,
		we're using a dictionary to avoid endless elif statements.
		"""
        	if msgID in self.Handlers.keys():
			self.Handlers[msgID](msgID,data)
	    	else:
			print "Unknown msgID: %d" % msgID
			print data
		return        

	def sendMsgAuth(self):
		pkg = PyDatagram()
		pkg.addUint16(CMSG_AUTH)
		pkg.addString(self.user.username)
		pkg.addString(self.user.pwd)
		pkg.addUint16(self.user.sector)
		pkg.addString(self.user.modelType)
		pkg.addString(self.user.remoteURL)
		pkg.addUint16(self.user.avatar.rX)
		pkg.addUint16(self.user.avatar.rY)
		pkg.addFloat32(self.user.avatar.getX())
		pkg.addFloat32(self.user.avatar.getY())
		pkg.addFloat32(self.user.avatar.getZ())
		pkg.addString(self.user.localIP)
		
		self.send(pkg)

	def sendMsgPos(self,sectorChange):
		pkg = PyDatagram()
		pkg.addUint16(CMSG_POS)
		if (sectorChange==True):
			flag=1
		else:
			flag=0
		pkg.addUint32(flag)
		pkg.addString(self.user.username)
		pkg.addUint16(self.user.sector)
		pkg.addUint16(self.user.avatar.rX)
		pkg.addUint16(self.user.avatar.rY)
		pkg.addFloat32(self.user.avatar.getX())
		pkg.addFloat32(self.user.avatar.getY())
		pkg.addFloat32(self.user.avatar.getZ())
		print "Sending SEC:%s X:%s Y:%s posX:%s posY:%s posZ:%s" % ( str(self.user.sector),str(self.user.avatar.rX),str(self.user.avatar.rY),str(self.user.avatar.getX()),str(self.user.avatar.getY()),str(self.user.avatar.getZ()) )
		self.send(pkg)
	
	def msgSectorModel(self, msgID, data):
		model={	'type':data.getString(),
			'data':data.getString()                                                      
			}
		#print "MODEL INFO: TYPE:%s \tDATA:%s" % ( model['type'],model['data'] )	
		print model['data']
		model['data']=eval(model['data'])
		
		modelThread=threading.Thread(	name='ModelThread'+model['type']+'-'+str(model['data']['x'])+'-'+str(model['data']['y'])+'-'+str(model['data']['z']),
							target=self.user.avatar.terrain.drawModel,kwargs={'model':model})
		modelThread.start()	
		#self.user.avatar.terrain.drawModel(model)
		if model['type']=='Tree':
			if (float(model['data']['z'])>self.user.avatar.getZ()):
				print 'Tree seems too high:',float(model['data']['z'])-self.user.avatar.getZ()
				print 'Tree Height:',float(model['data']['z']),'Avatar Height:',self.user.avatar.getZ()
			elif (self.user.avatar.getZ()-float(model['data']['z'])> 3):
				print 'Tree seems too low:',self.user.avatar.getZ()-float(model['data']['z'])
				print 'Tree Height:',float(model['data']['z']),'Avatar Height:',self.user.avatar.getZ()

	def msgSectorUser(self, msgID, data):
		user={	'username':data.getString(),
			#'IP':data.getString(),
			'modelType':data.getString(),
			'remoteURL':data.getString(),
			'sector':data.getUint16(),
			'posX':data.getFloat32(),
			'posY':data.getFloat32(),
			'posZ':data.getFloat32(),
			'client':data.getString(),
			'localIP':data.getString(),
			}
		print "USER INFO: UNAME:%s \tMODTYPE:%s \tURL:%s \tPOS:%s %s %s" % ( user['username'],user['modelType'],user['remoteURL'],str(user['posX']),str(user['posY']),str(user['posZ']) ) 	
		self.user.addOther(user)
	
	def msgPosResponse(self, msgID, data):
		flag = data.getUint32()
		if flag==0:
			print "bad position info sent"
		elif flag==1:
			print "position updated. awaiting models..."
			
	def sendMsgDisconnectReq(self):
		if self.connected==True:
			pkg = PyDatagram()
			pkg.addUint16(CMSG_DISCONNECT_REQ)
			pkg.addString(self.user.username)
			self.send(pkg)

	def msgAuthResponse(self, msgID, data):
		flag = data.getUint32()
		self.user.myIP=data.getString()
		if flag == 0:
			print "Unknown user"
		if flag == 1:
			print "Authentication Successful"
			self.connected=True
			pkg = PyDatagram()
			pkg.addUint16(CMSG_CHAT)
			pkg.addString("%s is calling in and is glad to be here" % self.user.username)
			self.send(pkg)
		    
		if flag == 2:
			print "Wrong pass, please try again..."
	    
	def msgPosResponse(self,msgID,data):
		flag = data.getUint32()
		print msgID,data
		
	def msgChat(self, msgID, data):
		print data.getString()

	def msgDisconnectAck(self, msgID, data):
		self.cManager.closeConnection(self.Connection)
		#sys.exit()

	def send(self, pkg):
		self.cWriter.send(pkg, self.Connection)

	def quit(self):
		self.cManager.closeConnection(self.Connection)
		sys.exit()
        
#aClient = Client()

#run()
